package ShowXPath.TreeNode;

import java.util.*;
import java.io.*;

import ShowXPath.StackNode.*;

public class LElement extends LTreeNode
{
	private LQname _qname;
	private LTreeNode _parent;
	private LNodeset _children = new LNodeset();
	private LNodeset _attributes = new LNodeset();
	private LNodeset _namespaces = new LNodeset();
	private String _original_name;
	private static String _type = "LElement";
	private String _lang = "";

	public String getLang()
	{
		if (_lang.equals(""))
		{
			return _parent.getLang();
		}
		else
		{
			return _lang;
		}
	}
	
	public LRoot getRoot()
	{
		if (_parent != null)
		{
			return _parent.getRoot();
		}
		else
		{
			return null;
		}
	}

	public String type()
	{
		return _type;
	}

	public LElement(String name, LTreeNode parent, Hashtable spaces)
	{
		_original_name = name;
		_qname = new LQname(name, spaces);
		_parent = parent;
		setID();
	}

	public LTreeNode getParent()
	{
		return _parent;
	}

	private void print(String string)
	{
		//System.out.println("In LElement: " + string);
	}

	public LNodeset getChildren()
	{
		print("In LElement getchildren()");
		if (_children != null)
		{
			print("Children is not empty");
		}
		else
		{
			print("Children is empty");
		}
		return _children;
	}
	
	public String getStringValue()
	{
		LNodeset nodes = descendent();
		Iterator i = nodes.listIterator();
		String s = "";
		while (i.hasNext())
		{
			Object o = i.next();
			if (o instanceof LText)
			{
				s = s + ((LText)o).getStringValue();
			}
		}
		return s;
	}
	
	public String getName()
	{
		return _qname.getLocalName() + ":" + _qname.getURI();
	}
	
	public String getLocal_name()
	{
		return _qname.getLocalName();
	}

	public String getNamespace()
	{
		return _qname.getNamespace();
	}

	public String getNamespace_uri()
	{
		return _qname.getURI();
	}

	public void setLang(String lang)
	{
		_lang = lang;
	}
	
	public void addAttribute(LAttribute a)
	{
		_attributes.addLast(a);
	}

	public void addNamespace(LNamespace a)
	{
		_namespaces.addLast(a);
	}

	public void add(LTreeNode n)
	{
		_children.addLast(n);
	}

	public String toString()
	{
		return "<" + _original_name + ">";
	}
	
	public String getNodeStringValue()
	{
		String s = "<" + _original_name + "{" + _order + "}";
		ListIterator i = _namespaces.listIterator();
		while (i.hasNext())
		{
			s = s + " " + ((LAttribute)i.next()).toString();
		}
		i = _attributes.listIterator();
		while (i.hasNext())
		{
			s = s + " " + ((LAttribute)i.next()).toString();
		}
		s = s + ">";
		i = _children.listIterator();
		while (i.hasNext())
		{
			s = s + ((LTreeNode)i.next()).toString();
		}
		s = s + "</" + _original_name + ">";
		return s;
	}

	public LNodeset parent()
	{
		LNodeset list = new LNodeset();
		list.add(_parent);
		return list;
	}

	public LNodeset child()
	{
		LNodeset _elements = new LNodeset();
		_elements.addAll(_children);
		return _elements;
	}

	public LNodeset ancestor()
	{
		LNodeset list = _parent.ancestor_or_self();
		return list;
	}

	public LNodeset self()
	{
		LNodeset list = new LNodeset();
		list.add(this);
		return list;
	}

	public LNodeset ancestor_or_self()
	{
		LNodeset list = _parent.ancestor_or_self();
		list.add(this);
		return list;
	}

	public LNodeset descendent()
	{
		LNodeset list = new LNodeset();
		list.addAll(_children);
		ListIterator i = _children.listIterator();
		while (i.hasNext())
		{
			list.addAll(((LTreeNode)i.next()).descendent());
		}
		return list;
	}

	public LNodeset descendent_or_self()
	{
		LNodeset list = descendent();
		list.add(this);
		return list;
	}

	public LNodeset attribute()
	{
		return _attributes;
	}

	public LNodeset following()
	{
		LNodeset list = root();
		list = ((LTreeNode)list.getFirst()).descendent();
		ListIterator i = list.listIterator();
		while (i.hasNext())
		{
			LTreeNode n = (LTreeNode)i.next();
			if (n.getOrder() <= _order)
			{
				i.remove();
			}
		}
		return list;
	}

	public LNodeset following_sibling()
	{
		LNodeset list = _parent.child();
		Iterator i = list.listIterator();
		while (i.hasNext())
		{
			LTreeNode n = (LTreeNode)i.next();
			if (n.getOrder() <= _order)
			{
				i.remove();
			}
		}
		return list;
	}

	public LNodeset preceding()
	{
		LNodeset list = root();
		list = ((LTreeNode)list.getFirst()).descendent();
		ListIterator i = list.listIterator();
		while (i.hasNext())
		{
			LTreeNode n = (LTreeNode)i.next();
			if (n.getOrder() >= _order)
			{
				i.remove();
			}
		}
		return list;
	}

	public LNodeset preceding_sibling()
	{
		LNodeset list = _parent.child();
		Iterator i = list.listIterator();
		while (i.hasNext())
		{
			LTreeNode n = (LTreeNode)i.next();
			if (n.getOrder() >= _order)
			{
				i.remove();
			}
		}
		return list;
	}

	public LNodeset namespace()
	{
		LNodeset list = new LNodeset();
		list.addAll(_namespaces);
		return list;
	}

	public String showNode()
	{
		return "<" + _original_name + "{" + _order + "}" + ">";
	}
	
}
